from .base_classes import *
from .support_functions import *


class Procedure(Resource):
    """An action that is performed on a patient. This can be a physical
    'thing' like an operation, or less invasive like counseling or
    hypnotherapy.If the element is present, it must have either a
    @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, subject=None, type_=None, bodySite=None, indication=None, performer=None, date=None, encounter=None, outcome=None, report=None, complication=None, followUp=None, relatedItem=None, notes=None):
        self.original_tagname_ = None
        super(Procedure, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.subject = subject
        self.type_ = type_
        if bodySite is None:
            self.bodySite = []
        else:
            self.bodySite = bodySite
        if indication is None:
            self.indication = []
        else:
            self.indication = indication
        if performer is None:
            self.performer = []
        else:
            self.performer = performer
        self.date = date
        self.encounter = encounter
        self.outcome = outcome
        if report is None:
            self.report = []
        else:
            self.report = report
        if complication is None:
            self.complication = []
        else:
            self.complication = complication
        self.followUp = followUp
        if relatedItem is None:
            self.relatedItem = []
        else:
            self.relatedItem = relatedItem
        self.notes = notes
    def factory(*args_, **kwargs_):
        if Procedure.subclass:
            return Procedure.subclass(*args_, **kwargs_)
        else:
            return Procedure(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_bodySite(self): return self.bodySite
    def set_bodySite(self, bodySite): self.bodySite = bodySite
    def add_bodySite(self, value): self.bodySite.append(value)
    def insert_bodySite(self, index, value): self.bodySite[index] = value
    def get_indication(self): return self.indication
    def set_indication(self, indication): self.indication = indication
    def add_indication(self, value): self.indication.append(value)
    def insert_indication(self, index, value): self.indication[index] = value
    def get_performer(self): return self.performer
    def set_performer(self, performer): self.performer = performer
    def add_performer(self, value): self.performer.append(value)
    def insert_performer(self, index, value): self.performer[index] = value
    def get_date(self): return self.date
    def set_date(self, date): self.date = date
    def get_encounter(self): return self.encounter
    def set_encounter(self, encounter): self.encounter = encounter
    def get_outcome(self): return self.outcome
    def set_outcome(self, outcome): self.outcome = outcome
    def get_report(self): return self.report
    def set_report(self, report): self.report = report
    def add_report(self, value): self.report.append(value)
    def insert_report(self, index, value): self.report[index] = value
    def get_complication(self): return self.complication
    def set_complication(self, complication): self.complication = complication
    def add_complication(self, value): self.complication.append(value)
    def insert_complication(self, index, value): self.complication[index] = value
    def get_followUp(self): return self.followUp
    def set_followUp(self, followUp): self.followUp = followUp
    def get_relatedItem(self): return self.relatedItem
    def set_relatedItem(self, relatedItem): self.relatedItem = relatedItem
    def add_relatedItem(self, value): self.relatedItem.append(value)
    def insert_relatedItem(self, index, value): self.relatedItem[index] = value
    def get_notes(self): return self.notes
    def set_notes(self, notes): self.notes = notes
    def hasContent_(self):
        if (
            self.identifier or
            self.subject is not None or
            self.type_ is not None or
            self.bodySite or
            self.indication or
            self.performer or
            self.date is not None or
            self.encounter is not None or
            self.outcome is not None or
            self.report or
            self.complication or
            self.followUp is not None or
            self.relatedItem or
            self.notes is not None or
            super(Procedure, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Procedure', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Procedure')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Procedure', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Procedure'):
        super(Procedure, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Procedure')
    def exportChildren(self, outfile, level, namespace_='', name_='Procedure', fromsubclass_=False, pretty_print=True):
        super(Procedure, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        for bodySite_ in self.bodySite:
            bodySite_.export(outfile, level, namespace_, name_='bodySite', pretty_print=pretty_print)
        for indication_ in self.indication:
            indication_.export(outfile, level, namespace_, name_='indication', pretty_print=pretty_print)
        for performer_ in self.performer:
            performer_.export(outfile, level, namespace_, name_='performer', pretty_print=pretty_print)
        if self.date is not None:
            self.date.export(outfile, level, namespace_, name_='date', pretty_print=pretty_print)
        if self.encounter is not None:
            self.encounter.export(outfile, level, namespace_, name_='encounter', pretty_print=pretty_print)
        if self.outcome is not None:
            self.outcome.export(outfile, level, namespace_, name_='outcome', pretty_print=pretty_print)
        for report_ in self.report:
            report_.export(outfile, level, namespace_, name_='report', pretty_print=pretty_print)
        for complication_ in self.complication:
            complication_.export(outfile, level, namespace_, name_='complication', pretty_print=pretty_print)
        if self.followUp is not None:
            self.followUp.export(outfile, level, namespace_, name_='followUp', pretty_print=pretty_print)
        for relatedItem_ in self.relatedItem:
            relatedItem_.export(outfile, level, namespace_, name_='relatedItem', pretty_print=pretty_print)
        if self.notes is not None:
            self.notes.export(outfile, level, namespace_, name_='notes', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Procedure', mapping_=None):
        element = super(Procedure, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        for bodySite_ in self.bodySite:
            bodySite_.to_etree(element, name_='bodySite', mapping_=mapping_)
        for indication_ in self.indication:
            indication_.to_etree(element, name_='indication', mapping_=mapping_)
        for performer_ in self.performer:
            performer_.to_etree(element, name_='performer', mapping_=mapping_)
        if self.date is not None:
            date_ = self.date
            date_.to_etree(element, name_='date', mapping_=mapping_)
        if self.encounter is not None:
            encounter_ = self.encounter
            encounter_.to_etree(element, name_='encounter', mapping_=mapping_)
        if self.outcome is not None:
            outcome_ = self.outcome
            outcome_.to_etree(element, name_='outcome', mapping_=mapping_)
        for report_ in self.report:
            report_.to_etree(element, name_='report', mapping_=mapping_)
        for complication_ in self.complication:
            complication_.to_etree(element, name_='complication', mapping_=mapping_)
        if self.followUp is not None:
            followUp_ = self.followUp
            followUp_.to_etree(element, name_='followUp', mapping_=mapping_)
        for relatedItem_ in self.relatedItem:
            relatedItem_.to_etree(element, name_='relatedItem', mapping_=mapping_)
        if self.notes is not None:
            notes_ = self.notes
            notes_.to_etree(element, name_='notes', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Procedure, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'type':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'bodySite':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.bodySite.append(obj_)
            obj_.original_tagname_ = 'bodySite'
        elif nodeName_ == 'indication':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.indication.append(obj_)
            obj_.original_tagname_ = 'indication'
        elif nodeName_ == 'performer':
            obj_ = Procedure_Performer.factory()
            obj_.build(child_)
            self.performer.append(obj_)
            obj_.original_tagname_ = 'performer'
        elif nodeName_ == 'date':
            obj_ = Period.factory()
            obj_.build(child_)
            self.date = obj_
            obj_.original_tagname_ = 'date'
        elif nodeName_ == 'encounter':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.encounter = obj_
            obj_.original_tagname_ = 'encounter'
        elif nodeName_ == 'outcome':
            obj_ = string.factory()
            obj_.build(child_)
            self.outcome = obj_
            obj_.original_tagname_ = 'outcome'
        elif nodeName_ == 'report':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.report.append(obj_)
            obj_.original_tagname_ = 'report'
        elif nodeName_ == 'complication':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.complication.append(obj_)
            obj_.original_tagname_ = 'complication'
        elif nodeName_ == 'followUp':
            obj_ = string.factory()
            obj_.build(child_)
            self.followUp = obj_
            obj_.original_tagname_ = 'followUp'
        elif nodeName_ == 'relatedItem':
            obj_ = Procedure_RelatedItem.factory()
            obj_.build(child_)
            self.relatedItem.append(obj_)
            obj_.original_tagname_ = 'relatedItem'
        elif nodeName_ == 'notes':
            obj_ = string.factory()
            obj_.build(child_)
            self.notes = obj_
            obj_.original_tagname_ = 'notes'
        super(Procedure, self).buildChildren(child_, node, nodeName_, True)
# end class Procedure


class Procedure_Performer(BackboneElement):
    """An action that is performed on a patient. This can be a physical
    'thing' like an operation, or less invasive like counseling or
    hypnotherapy."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, person=None, role=None):
        self.original_tagname_ = None
        super(Procedure_Performer, self).__init__(id, extension, modifierExtension, )
        self.person = person
        self.role = role
    def factory(*args_, **kwargs_):
        if Procedure_Performer.subclass:
            return Procedure_Performer.subclass(*args_, **kwargs_)
        else:
            return Procedure_Performer(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_person(self): return self.person
    def set_person(self, person): self.person = person
    def get_role(self): return self.role
    def set_role(self, role): self.role = role
    def hasContent_(self):
        if (
            self.person is not None or
            self.role is not None or
            super(Procedure_Performer, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Procedure.Performer', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Procedure.Performer')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Procedure.Performer', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Procedure.Performer'):
        super(Procedure_Performer, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Procedure.Performer')
    def exportChildren(self, outfile, level, namespace_='', name_='Procedure.Performer', fromsubclass_=False, pretty_print=True):
        super(Procedure_Performer, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.person is not None:
            self.person.export(outfile, level, namespace_, name_='person', pretty_print=pretty_print)
        if self.role is not None:
            self.role.export(outfile, level, namespace_, name_='role', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Procedure.Performer', mapping_=None):
        element = super(Procedure_Performer, self).to_etree(parent_element, name_, mapping_)
        if self.person is not None:
            person_ = self.person
            person_.to_etree(element, name_='person', mapping_=mapping_)
        if self.role is not None:
            role_ = self.role
            role_.to_etree(element, name_='role', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Procedure_Performer, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'person':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.person = obj_
            obj_.original_tagname_ = 'person'
        elif nodeName_ == 'role':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.role = obj_
            obj_.original_tagname_ = 'role'
        super(Procedure_Performer, self).buildChildren(child_, node, nodeName_, True)
# end class Procedure_Performer


class Procedure_RelatedItem(BackboneElement):
    """An action that is performed on a patient. This can be a physical
    'thing' like an operation, or less invasive like counseling or
    hypnotherapy."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, type_=None, target=None):
        self.original_tagname_ = None
        super(Procedure_RelatedItem, self).__init__(id, extension, modifierExtension, )
        self.type_ = type_
        self.target = target
    def factory(*args_, **kwargs_):
        if Procedure_RelatedItem.subclass:
            return Procedure_RelatedItem.subclass(*args_, **kwargs_)
        else:
            return Procedure_RelatedItem(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_type(self): return self.type_
    def set_type(self, type_): self.type_ = type_
    def get_target(self): return self.target
    def set_target(self, target): self.target = target
    def hasContent_(self):
        if (
            self.type_ is not None or
            self.target is not None or
            super(Procedure_RelatedItem, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Procedure.RelatedItem', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Procedure.RelatedItem')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Procedure.RelatedItem', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Procedure.RelatedItem'):
        super(Procedure_RelatedItem, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Procedure.RelatedItem')
    def exportChildren(self, outfile, level, namespace_='', name_='Procedure.RelatedItem', fromsubclass_=False, pretty_print=True):
        super(Procedure_RelatedItem, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.type_ is not None:
            self.type_.export(outfile, level, namespace_, name_='type', pretty_print=pretty_print)
        if self.target is not None:
            self.target.export(outfile, level, namespace_, name_='target', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Procedure.RelatedItem', mapping_=None):
        element = super(Procedure_RelatedItem, self).to_etree(parent_element, name_, mapping_)
        if self.type_ is not None:
            type__ = self.type_
            type__.to_etree(element, name_='type', mapping_=mapping_)
        if self.target is not None:
            target_ = self.target
            target_.to_etree(element, name_='target', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Procedure_RelatedItem, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'type':
            obj_ = ProcedureRelationshipType.factory()
            obj_.build(child_)
            self.type_ = obj_
            obj_.original_tagname_ = 'type'
        elif nodeName_ == 'target':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.target = obj_
            obj_.original_tagname_ = 'target'
        super(Procedure_RelatedItem, self).buildChildren(child_, node, nodeName_, True)
# end class Procedure_RelatedItem


class ProcedureRelationshipType(Element):
    """The nature of the relationship with this procedureIf the element is
    present, it must have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(ProcedureRelationshipType, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if ProcedureRelationshipType.subclass:
            return ProcedureRelationshipType.subclass(*args_, **kwargs_)
        else:
            return ProcedureRelationshipType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(ProcedureRelationshipType, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ProcedureRelationshipType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ProcedureRelationshipType')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ProcedureRelationshipType', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ProcedureRelationshipType'):
        super(ProcedureRelationshipType, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ProcedureRelationshipType')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='ProcedureRelationshipType', fromsubclass_=False, pretty_print=True):
        super(ProcedureRelationshipType, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ProcedureRelationshipType', mapping_=None):
        element = super(ProcedureRelationshipType, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(ProcedureRelationshipType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(ProcedureRelationshipType, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class ProcedureRelationshipType



from .base_classes import *
from .support_functions import *


class Immunization(Resource):
    """Immunization event information.If the element is present, it must
    have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, date=None, vaccineType=None, subject=None, refusedIndicator=None, reported=None, performer=None, requester=None, manufacturer=None, location=None, lotNumber=None, expirationDate=None, site=None, route=None, doseQuantity=None, explanation=None, reaction=None, vaccinationProtocol=None):
        self.original_tagname_ = None
        super(Immunization, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.date = date
        self.vaccineType = vaccineType
        self.subject = subject
        self.refusedIndicator = refusedIndicator
        self.reported = reported
        self.performer = performer
        self.requester = requester
        self.manufacturer = manufacturer
        self.location = location
        self.lotNumber = lotNumber
        self.expirationDate = expirationDate
        self.site = site
        self.route = route
        self.doseQuantity = doseQuantity
        self.explanation = explanation
        if reaction is None:
            self.reaction = []
        else:
            self.reaction = reaction
        if vaccinationProtocol is None:
            self.vaccinationProtocol = []
        else:
            self.vaccinationProtocol = vaccinationProtocol
    def factory(*args_, **kwargs_):
        if Immunization.subclass:
            return Immunization.subclass(*args_, **kwargs_)
        else:
            return Immunization(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_date(self): return self.date
    def set_date(self, date): self.date = date
    def get_vaccineType(self): return self.vaccineType
    def set_vaccineType(self, vaccineType): self.vaccineType = vaccineType
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_refusedIndicator(self): return self.refusedIndicator
    def set_refusedIndicator(self, refusedIndicator): self.refusedIndicator = refusedIndicator
    def get_reported(self): return self.reported
    def set_reported(self, reported): self.reported = reported
    def get_performer(self): return self.performer
    def set_performer(self, performer): self.performer = performer
    def get_requester(self): return self.requester
    def set_requester(self, requester): self.requester = requester
    def get_manufacturer(self): return self.manufacturer
    def set_manufacturer(self, manufacturer): self.manufacturer = manufacturer
    def get_location(self): return self.location
    def set_location(self, location): self.location = location
    def get_lotNumber(self): return self.lotNumber
    def set_lotNumber(self, lotNumber): self.lotNumber = lotNumber
    def get_expirationDate(self): return self.expirationDate
    def set_expirationDate(self, expirationDate): self.expirationDate = expirationDate
    def get_site(self): return self.site
    def set_site(self, site): self.site = site
    def get_route(self): return self.route
    def set_route(self, route): self.route = route
    def get_doseQuantity(self): return self.doseQuantity
    def set_doseQuantity(self, doseQuantity): self.doseQuantity = doseQuantity
    def get_explanation(self): return self.explanation
    def set_explanation(self, explanation): self.explanation = explanation
    def get_reaction(self): return self.reaction
    def set_reaction(self, reaction): self.reaction = reaction
    def add_reaction(self, value): self.reaction.append(value)
    def insert_reaction(self, index, value): self.reaction[index] = value
    def get_vaccinationProtocol(self): return self.vaccinationProtocol
    def set_vaccinationProtocol(self, vaccinationProtocol): self.vaccinationProtocol = vaccinationProtocol
    def add_vaccinationProtocol(self, value): self.vaccinationProtocol.append(value)
    def insert_vaccinationProtocol(self, index, value): self.vaccinationProtocol[index] = value
    def hasContent_(self):
        if (
            self.identifier or
            self.date is not None or
            self.vaccineType is not None or
            self.subject is not None or
            self.refusedIndicator is not None or
            self.reported is not None or
            self.performer is not None or
            self.requester is not None or
            self.manufacturer is not None or
            self.location is not None or
            self.lotNumber is not None or
            self.expirationDate is not None or
            self.site is not None or
            self.route is not None or
            self.doseQuantity is not None or
            self.explanation is not None or
            self.reaction or
            self.vaccinationProtocol or
            super(Immunization, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Immunization', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Immunization')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Immunization', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Immunization'):
        super(Immunization, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Immunization')
    def exportChildren(self, outfile, level, namespace_='', name_='Immunization', fromsubclass_=False, pretty_print=True):
        super(Immunization, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.date is not None:
            self.date.export(outfile, level, namespace_, name_='date', pretty_print=pretty_print)
        if self.vaccineType is not None:
            self.vaccineType.export(outfile, level, namespace_, name_='vaccineType', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.refusedIndicator is not None:
            self.refusedIndicator.export(outfile, level, namespace_, name_='refusedIndicator', pretty_print=pretty_print)
        if self.reported is not None:
            self.reported.export(outfile, level, namespace_, name_='reported', pretty_print=pretty_print)
        if self.performer is not None:
            self.performer.export(outfile, level, namespace_, name_='performer', pretty_print=pretty_print)
        if self.requester is not None:
            self.requester.export(outfile, level, namespace_, name_='requester', pretty_print=pretty_print)
        if self.manufacturer is not None:
            self.manufacturer.export(outfile, level, namespace_, name_='manufacturer', pretty_print=pretty_print)
        if self.location is not None:
            self.location.export(outfile, level, namespace_, name_='location', pretty_print=pretty_print)
        if self.lotNumber is not None:
            self.lotNumber.export(outfile, level, namespace_, name_='lotNumber', pretty_print=pretty_print)
        if self.expirationDate is not None:
            self.expirationDate.export(outfile, level, namespace_, name_='expirationDate', pretty_print=pretty_print)
        if self.site is not None:
            self.site.export(outfile, level, namespace_, name_='site', pretty_print=pretty_print)
        if self.route is not None:
            self.route.export(outfile, level, namespace_, name_='route', pretty_print=pretty_print)
        if self.doseQuantity is not None:
            self.doseQuantity.export(outfile, level, namespace_, name_='doseQuantity', pretty_print=pretty_print)
        if self.explanation is not None:
            self.explanation.export(outfile, level, namespace_, name_='explanation', pretty_print=pretty_print)
        for reaction_ in self.reaction:
            reaction_.export(outfile, level, namespace_, name_='reaction', pretty_print=pretty_print)
        for vaccinationProtocol_ in self.vaccinationProtocol:
            vaccinationProtocol_.export(outfile, level, namespace_, name_='vaccinationProtocol', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Immunization', mapping_=None):
        element = super(Immunization, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.date is not None:
            date_ = self.date
            date_.to_etree(element, name_='date', mapping_=mapping_)
        if self.vaccineType is not None:
            vaccineType_ = self.vaccineType
            vaccineType_.to_etree(element, name_='vaccineType', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.refusedIndicator is not None:
            refusedIndicator_ = self.refusedIndicator
            refusedIndicator_.to_etree(element, name_='refusedIndicator', mapping_=mapping_)
        if self.reported is not None:
            reported_ = self.reported
            reported_.to_etree(element, name_='reported', mapping_=mapping_)
        if self.performer is not None:
            performer_ = self.performer
            performer_.to_etree(element, name_='performer', mapping_=mapping_)
        if self.requester is not None:
            requester_ = self.requester
            requester_.to_etree(element, name_='requester', mapping_=mapping_)
        if self.manufacturer is not None:
            manufacturer_ = self.manufacturer
            manufacturer_.to_etree(element, name_='manufacturer', mapping_=mapping_)
        if self.location is not None:
            location_ = self.location
            location_.to_etree(element, name_='location', mapping_=mapping_)
        if self.lotNumber is not None:
            lotNumber_ = self.lotNumber
            lotNumber_.to_etree(element, name_='lotNumber', mapping_=mapping_)
        if self.expirationDate is not None:
            expirationDate_ = self.expirationDate
            expirationDate_.to_etree(element, name_='expirationDate', mapping_=mapping_)
        if self.site is not None:
            site_ = self.site
            site_.to_etree(element, name_='site', mapping_=mapping_)
        if self.route is not None:
            route_ = self.route
            route_.to_etree(element, name_='route', mapping_=mapping_)
        if self.doseQuantity is not None:
            doseQuantity_ = self.doseQuantity
            doseQuantity_.to_etree(element, name_='doseQuantity', mapping_=mapping_)
        if self.explanation is not None:
            explanation_ = self.explanation
            explanation_.to_etree(element, name_='explanation', mapping_=mapping_)
        for reaction_ in self.reaction:
            reaction_.to_etree(element, name_='reaction', mapping_=mapping_)
        for vaccinationProtocol_ in self.vaccinationProtocol:
            vaccinationProtocol_.to_etree(element, name_='vaccinationProtocol', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Immunization, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'date':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.date = obj_
            obj_.original_tagname_ = 'date'
        elif nodeName_ == 'vaccineType':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.vaccineType = obj_
            obj_.original_tagname_ = 'vaccineType'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'refusedIndicator':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.refusedIndicator = obj_
            obj_.original_tagname_ = 'refusedIndicator'
        elif nodeName_ == 'reported':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.reported = obj_
            obj_.original_tagname_ = 'reported'
        elif nodeName_ == 'performer':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.performer = obj_
            obj_.original_tagname_ = 'performer'
        elif nodeName_ == 'requester':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.requester = obj_
            obj_.original_tagname_ = 'requester'
        elif nodeName_ == 'manufacturer':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.manufacturer = obj_
            obj_.original_tagname_ = 'manufacturer'
        elif nodeName_ == 'location':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.location = obj_
            obj_.original_tagname_ = 'location'
        elif nodeName_ == 'lotNumber':
            obj_ = string.factory()
            obj_.build(child_)
            self.lotNumber = obj_
            obj_.original_tagname_ = 'lotNumber'
        elif nodeName_ == 'expirationDate':
            obj_ = date.factory()
            obj_.build(child_)
            self.expirationDate = obj_
            obj_.original_tagname_ = 'expirationDate'
        elif nodeName_ == 'site':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.site = obj_
            obj_.original_tagname_ = 'site'
        elif nodeName_ == 'route':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.route = obj_
            obj_.original_tagname_ = 'route'
        elif nodeName_ == 'doseQuantity':
            obj_ = Quantity.factory()
            obj_.build(child_)
            self.doseQuantity = obj_
            obj_.original_tagname_ = 'doseQuantity'
        elif nodeName_ == 'explanation':
            obj_ = Immunization_Explanation.factory()
            obj_.build(child_)
            self.explanation = obj_
            obj_.original_tagname_ = 'explanation'
        elif nodeName_ == 'reaction':
            obj_ = Immunization_Reaction.factory()
            obj_.build(child_)
            self.reaction.append(obj_)
            obj_.original_tagname_ = 'reaction'
        elif nodeName_ == 'vaccinationProtocol':
            obj_ = Immunization_VaccinationProtocol.factory()
            obj_.build(child_)
            self.vaccinationProtocol.append(obj_)
            obj_.original_tagname_ = 'vaccinationProtocol'
        super(Immunization, self).buildChildren(child_, node, nodeName_, True)
# end class Immunization


class Immunization_Explanation(BackboneElement):
    """Immunization event information."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, reason=None, refusalReason=None):
        self.original_tagname_ = None
        super(Immunization_Explanation, self).__init__(id, extension, modifierExtension, )
        if reason is None:
            self.reason = []
        else:
            self.reason = reason
        if refusalReason is None:
            self.refusalReason = []
        else:
            self.refusalReason = refusalReason
    def factory(*args_, **kwargs_):
        if Immunization_Explanation.subclass:
            return Immunization_Explanation.subclass(*args_, **kwargs_)
        else:
            return Immunization_Explanation(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_reason(self): return self.reason
    def set_reason(self, reason): self.reason = reason
    def add_reason(self, value): self.reason.append(value)
    def insert_reason(self, index, value): self.reason[index] = value
    def get_refusalReason(self): return self.refusalReason
    def set_refusalReason(self, refusalReason): self.refusalReason = refusalReason
    def add_refusalReason(self, value): self.refusalReason.append(value)
    def insert_refusalReason(self, index, value): self.refusalReason[index] = value
    def hasContent_(self):
        if (
            self.reason or
            self.refusalReason or
            super(Immunization_Explanation, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Immunization.Explanation', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Immunization.Explanation')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Immunization.Explanation', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Immunization.Explanation'):
        super(Immunization_Explanation, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Immunization.Explanation')
    def exportChildren(self, outfile, level, namespace_='', name_='Immunization.Explanation', fromsubclass_=False, pretty_print=True):
        super(Immunization_Explanation, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for reason_ in self.reason:
            reason_.export(outfile, level, namespace_, name_='reason', pretty_print=pretty_print)
        for refusalReason_ in self.refusalReason:
            refusalReason_.export(outfile, level, namespace_, name_='refusalReason', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Immunization.Explanation', mapping_=None):
        element = super(Immunization_Explanation, self).to_etree(parent_element, name_, mapping_)
        for reason_ in self.reason:
            reason_.to_etree(element, name_='reason', mapping_=mapping_)
        for refusalReason_ in self.refusalReason:
            refusalReason_.to_etree(element, name_='refusalReason', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Immunization_Explanation, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'reason':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.reason.append(obj_)
            obj_.original_tagname_ = 'reason'
        elif nodeName_ == 'refusalReason':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.refusalReason.append(obj_)
            obj_.original_tagname_ = 'refusalReason'
        super(Immunization_Explanation, self).buildChildren(child_, node, nodeName_, True)
# end class Immunization_Explanation


class Immunization_Reaction(BackboneElement):
    """Immunization event information."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, date=None, detail=None, reported=None):
        self.original_tagname_ = None
        super(Immunization_Reaction, self).__init__(id, extension, modifierExtension, )
        self.date = date
        self.detail = detail
        self.reported = reported
    def factory(*args_, **kwargs_):
        if Immunization_Reaction.subclass:
            return Immunization_Reaction.subclass(*args_, **kwargs_)
        else:
            return Immunization_Reaction(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_date(self): return self.date
    def set_date(self, date): self.date = date
    def get_detail(self): return self.detail
    def set_detail(self, detail): self.detail = detail
    def get_reported(self): return self.reported
    def set_reported(self, reported): self.reported = reported
    def hasContent_(self):
        if (
            self.date is not None or
            self.detail is not None or
            self.reported is not None or
            super(Immunization_Reaction, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Immunization.Reaction', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Immunization.Reaction')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Immunization.Reaction', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Immunization.Reaction'):
        super(Immunization_Reaction, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Immunization.Reaction')
    def exportChildren(self, outfile, level, namespace_='', name_='Immunization.Reaction', fromsubclass_=False, pretty_print=True):
        super(Immunization_Reaction, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.date is not None:
            self.date.export(outfile, level, namespace_, name_='date', pretty_print=pretty_print)
        if self.detail is not None:
            self.detail.export(outfile, level, namespace_, name_='detail', pretty_print=pretty_print)
        if self.reported is not None:
            self.reported.export(outfile, level, namespace_, name_='reported', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Immunization.Reaction', mapping_=None):
        element = super(Immunization_Reaction, self).to_etree(parent_element, name_, mapping_)
        if self.date is not None:
            date_ = self.date
            date_.to_etree(element, name_='date', mapping_=mapping_)
        if self.detail is not None:
            detail_ = self.detail
            detail_.to_etree(element, name_='detail', mapping_=mapping_)
        if self.reported is not None:
            reported_ = self.reported
            reported_.to_etree(element, name_='reported', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Immunization_Reaction, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'date':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.date = obj_
            obj_.original_tagname_ = 'date'
        elif nodeName_ == 'detail':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.detail = obj_
            obj_.original_tagname_ = 'detail'
        elif nodeName_ == 'reported':
            obj_ = boolean.factory()
            obj_.build(child_)
            self.reported = obj_
            obj_.original_tagname_ = 'reported'
        super(Immunization_Reaction, self).buildChildren(child_, node, nodeName_, True)
# end class Immunization_Reaction


class Immunization_VaccinationProtocol(BackboneElement):
    """Immunization event information."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, doseSequence=None, description=None, authority=None, series=None, seriesDoses=None, doseTarget=None, doseStatus=None, doseStatusReason=None):
        self.original_tagname_ = None
        super(Immunization_VaccinationProtocol, self).__init__(id, extension, modifierExtension, )
        self.doseSequence = doseSequence
        self.description = description
        self.authority = authority
        self.series = series
        self.seriesDoses = seriesDoses
        self.doseTarget = doseTarget
        self.doseStatus = doseStatus
        self.doseStatusReason = doseStatusReason
    def factory(*args_, **kwargs_):
        if Immunization_VaccinationProtocol.subclass:
            return Immunization_VaccinationProtocol.subclass(*args_, **kwargs_)
        else:
            return Immunization_VaccinationProtocol(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_doseSequence(self): return self.doseSequence
    def set_doseSequence(self, doseSequence): self.doseSequence = doseSequence
    def get_description(self): return self.description
    def set_description(self, description): self.description = description
    def get_authority(self): return self.authority
    def set_authority(self, authority): self.authority = authority
    def get_series(self): return self.series
    def set_series(self, series): self.series = series
    def get_seriesDoses(self): return self.seriesDoses
    def set_seriesDoses(self, seriesDoses): self.seriesDoses = seriesDoses
    def get_doseTarget(self): return self.doseTarget
    def set_doseTarget(self, doseTarget): self.doseTarget = doseTarget
    def get_doseStatus(self): return self.doseStatus
    def set_doseStatus(self, doseStatus): self.doseStatus = doseStatus
    def get_doseStatusReason(self): return self.doseStatusReason
    def set_doseStatusReason(self, doseStatusReason): self.doseStatusReason = doseStatusReason
    def hasContent_(self):
        if (
            self.doseSequence is not None or
            self.description is not None or
            self.authority is not None or
            self.series is not None or
            self.seriesDoses is not None or
            self.doseTarget is not None or
            self.doseStatus is not None or
            self.doseStatusReason is not None or
            super(Immunization_VaccinationProtocol, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Immunization.VaccinationProtocol', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Immunization.VaccinationProtocol')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Immunization.VaccinationProtocol', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Immunization.VaccinationProtocol'):
        super(Immunization_VaccinationProtocol, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Immunization.VaccinationProtocol')
    def exportChildren(self, outfile, level, namespace_='', name_='Immunization.VaccinationProtocol', fromsubclass_=False, pretty_print=True):
        super(Immunization_VaccinationProtocol, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.doseSequence is not None:
            self.doseSequence.export(outfile, level, namespace_, name_='doseSequence', pretty_print=pretty_print)
        if self.description is not None:
            self.description.export(outfile, level, namespace_, name_='description', pretty_print=pretty_print)
        if self.authority is not None:
            self.authority.export(outfile, level, namespace_, name_='authority', pretty_print=pretty_print)
        if self.series is not None:
            self.series.export(outfile, level, namespace_, name_='series', pretty_print=pretty_print)
        if self.seriesDoses is not None:
            self.seriesDoses.export(outfile, level, namespace_, name_='seriesDoses', pretty_print=pretty_print)
        if self.doseTarget is not None:
            self.doseTarget.export(outfile, level, namespace_, name_='doseTarget', pretty_print=pretty_print)
        if self.doseStatus is not None:
            self.doseStatus.export(outfile, level, namespace_, name_='doseStatus', pretty_print=pretty_print)
        if self.doseStatusReason is not None:
            self.doseStatusReason.export(outfile, level, namespace_, name_='doseStatusReason', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Immunization.VaccinationProtocol', mapping_=None):
        element = super(Immunization_VaccinationProtocol, self).to_etree(parent_element, name_, mapping_)
        if self.doseSequence is not None:
            doseSequence_ = self.doseSequence
            doseSequence_.to_etree(element, name_='doseSequence', mapping_=mapping_)
        if self.description is not None:
            description_ = self.description
            description_.to_etree(element, name_='description', mapping_=mapping_)
        if self.authority is not None:
            authority_ = self.authority
            authority_.to_etree(element, name_='authority', mapping_=mapping_)
        if self.series is not None:
            series_ = self.series
            series_.to_etree(element, name_='series', mapping_=mapping_)
        if self.seriesDoses is not None:
            seriesDoses_ = self.seriesDoses
            seriesDoses_.to_etree(element, name_='seriesDoses', mapping_=mapping_)
        if self.doseTarget is not None:
            doseTarget_ = self.doseTarget
            doseTarget_.to_etree(element, name_='doseTarget', mapping_=mapping_)
        if self.doseStatus is not None:
            doseStatus_ = self.doseStatus
            doseStatus_.to_etree(element, name_='doseStatus', mapping_=mapping_)
        if self.doseStatusReason is not None:
            doseStatusReason_ = self.doseStatusReason
            doseStatusReason_.to_etree(element, name_='doseStatusReason', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Immunization_VaccinationProtocol, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'doseSequence':
            obj_ = integer.factory()
            obj_.build(child_)
            self.doseSequence = obj_
            obj_.original_tagname_ = 'doseSequence'
        elif nodeName_ == 'description':
            obj_ = string.factory()
            obj_.build(child_)
            self.description = obj_
            obj_.original_tagname_ = 'description'
        elif nodeName_ == 'authority':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.authority = obj_
            obj_.original_tagname_ = 'authority'
        elif nodeName_ == 'series':
            obj_ = string.factory()
            obj_.build(child_)
            self.series = obj_
            obj_.original_tagname_ = 'series'
        elif nodeName_ == 'seriesDoses':
            obj_ = integer.factory()
            obj_.build(child_)
            self.seriesDoses = obj_
            obj_.original_tagname_ = 'seriesDoses'
        elif nodeName_ == 'doseTarget':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.doseTarget = obj_
            obj_.original_tagname_ = 'doseTarget'
        elif nodeName_ == 'doseStatus':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.doseStatus = obj_
            obj_.original_tagname_ = 'doseStatus'
        elif nodeName_ == 'doseStatusReason':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.doseStatusReason = obj_
            obj_.original_tagname_ = 'doseStatusReason'
        super(Immunization_VaccinationProtocol, self).buildChildren(child_, node, nodeName_, True)
# end class Immunization_VaccinationProtocol



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <falcon@gnu.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#    Copyright (C) 2013  Sebastian Marro <smarro@gnusolidario.org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from trytond.wizard import Wizard, StateView, Button, StateTransition
from trytond.model import ModelView
from trytond.transaction import Transaction
from trytond.pool import Pool

__all__ = ['CreateVaccinationStockMoveInit','CreateVaccinationStockMove']

class CreateVaccinationStockMoveInit(ModelView):
    'Create Vaccination Stock Move Init'
    __name__ = 'gnuhealth.vaccination.stock.move.init'


class CreateVaccinationStockMove(Wizard):
    'Create Vaccination Stock Move'
    __name__ = 'gnuhealth.vaccination.stock.move.create'

    start = StateView('gnuhealth.vaccination.stock.move.init',
            'health_stock.view_create_vaccination_stock_move', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Create Stock Move', 'create_stock_move',
                'tryton-ok', True),
            ])
    create_stock_move = StateTransition()

    def transition_create_stock_move(self):
        pool = Pool()
        StockMove = pool.get('stock.move')
        Vaccination = pool.get('gnuhealth.vaccination')

        vaccinations = Vaccination.browse(Transaction().context.get(
            'active_ids'))
        for vaccination in vaccinations:

            if vaccination.moves:
                self.raise_user_error('stock_move_exists')
                 
            lines = []
            
            line_data = {}
            line_data['origin'] = str(vaccination)
            line_data['from_location'] = \
                vaccination.location.id
            line_data['to_location'] = \
                vaccination.name.name.customer_location.id
            line_data['product'] = \
                vaccination.vaccine.name.id
            line_data['unit_price'] = \
                vaccination.vaccine.name.list_price
            line_data['quantity'] = 1
            line_data['uom'] = \
                vaccination.vaccine.name.default_uom.id
            line_data['state'] = 'draft'
            lines.append(line_data)
            
            moves = StockMove.create(lines)

            StockMove.assign(moves)
            StockMove.do(moves)
            
            
        return 'end'

    @classmethod
    def __setup__(cls):
        super(CreateVaccinationStockMove, cls).__setup__()
        cls._error_messages.update({
            'stock_move_exists':
                'Stock moves already exists!.',})




import os
import logging
import types
import shutil
from io import StringIO
import urllib.request, urllib.parse, urllib.error
from pywebdav.lib.constants import COLLECTION, OBJECT
from pywebdav.lib.errors import *
from pywebdav.lib.iface import *
from pywebdav.lib.davcmd import copyone, copytree, moveone, movetree, delone, deltree

log = logging.getLogger(__name__)

BUFFER_SIZE = 128 * 1000 
# include magic support to correctly determine mimetypes
MAGIC_AVAILABLE = False
try:
    import mimetypes
    MAGIC_AVAILABLE = True
    log.info('Mimetype support ENABLED')
except ImportError:
    log.info('Mimetype support DISABLED')
    pass

class Resource(object):
    # XXX this class is ugly
    def __init__(self, fp, file_size):
        self.__fp = fp
        self.__file_size = file_size

    def __len__(self):
        return self.__file_size

    def __iter__(self):
        while 1:
            data = self.__fp.read(BUFFER_SIZE)
            if not data:
                break
            yield data
            time.sleep(0.005)
        self.__fp.close()

    def read(self, length = 0):
        if length == 0:
            length = self.__file_size

        data = self.__fp.read(length)
        return data
        

class FilesystemHandler(dav_interface):
    """ 
    Model a filesystem for DAV

    This class models a regular filesystem for the DAV server

    The basic URL will be http://localhost/
    And the underlying filesystem will be /tmp

    Thus http://localhost/gfx/pix will lead
    to /tmp/gfx/pix

    """

    def __init__(self, directory, uri, verbose=False):
        self.setDirectory(directory)
        self.setBaseURI(uri)

        # should we be verbose?
        self.verbose = verbose
        log.info('Initialized with %s %s' % (directory, uri))

    def setDirectory(self, path):
        """ Sets the directory """

        if not os.path.isdir(path):
            raise Exception('%s not must be a directory!' % path)

        self.directory = path

    def setBaseURI(self, uri):
        """ Sets the base uri """

        self.baseuri = uri

    def uri2local(self,uri):
        """ map uri in baseuri and local part """
        uparts=urllib.parse.urlparse(uri.decode())
        fileloc=uparts[2][1:]
        filename=os.path.join(self.directory, fileloc)
        filename=os.path.normpath(filename)
        return filename

    def local2uri(self,filename):
        """ map local filename to self.baseuri """

        pnum=len(self.directory.replace("\\","/").split("/"))
        parts=filename.replace("\\","/").split("/")[pnum:]
        sparts="/"+"/".join(parts)
        uri=urllib.parse.urljoin(self.baseuri,sparts)
        return uri.encode() if isinstance(uri, str) else uri


    def get_childs(self, uri, filter=None):
        """ return the child objects as self.baseuris for the given URI """

        fileloc=self.uri2local(uri)
        filelist=[]

        if os.path.exists(fileloc):
            if os.path.isdir(fileloc):
                try:
                    files=os.listdir(fileloc)
                except:
                    raise DAV_NotFound

                for file in files:
                    newloc=os.path.join(fileloc,file)
                    filelist.append(self.local2uri(newloc))

                log.info('get_childs: Childs %s' % filelist)

        return filelist

    def get_data(self,uri, range = None):
        """ return the content of an object """

        path=self.uri2local(uri)
        if os.path.exists(path):
            if os.path.isfile(path):
                file_size = os.path.getsize(path)
                if range is None:
                    fp=open(path,"r")
                    log.info('Serving content of %s' % uri)
                    return Resource(fp, file_size)
                else:
                    if range[1] == '':
                        range[1] = file_size
                    else:
                        range[1] = int(range[1])

                    if range[0] == '':
                        range[0] = file_size - range[1]
                    else:
                        range[0] = int(range[0])

                    if range[0] > file_size:
                        raise DAV_Requested_Range_Not_Satisfiable

                    if range[1] > file_size:
                        range[1] = file_size

                    fp=open(path,"r")
                    fp.seek(range[0])
                    log.info('Serving range %s -> %s content of %s' % (range[0], range[1], uri))
                    return Resource(fp, range[1] - range[0])
            elif os.path.isdir(path):
                # GET for collections is defined as 'return s/th meaningful' :-)
                msg = 'Directory at %s' % uri
                return Resource(StringIO(msg), len(msg))
            else:
                # also raise an error for collections
                # don't know what should happen then..
                log.info('get_data: %s not found' % path)

        raise DAV_NotFound

    def _get_dav_resourcetype(self,uri):
        """ return type of object """
        path=self.uri2local(uri)
        if os.path.isfile(path):
            return OBJECT

        elif os.path.isdir(path):
            return COLLECTION

        raise DAV_NotFound

    def _get_dav_displayname(self,uri):
        raise DAV_Secret    # do not show

    def _get_dav_getcontentlength(self,uri):
        """ return the content length of an object """
        path=self.uri2local(uri)
        if os.path.exists(path):
            if os.path.isfile(path):
                s=os.stat(path)
                return str(s[6])

        return '0'

    def get_lastmodified(self,uri):
        """ return the last modified date of the object """
        path=self.uri2local(uri)
        if os.path.exists(path):
            s=os.stat(path)
            date=s[8]
            return date

        raise DAV_NotFound

    def get_creationdate(self,uri):
        """ return the last modified date of the object """
        path=self.uri2local(uri)
        if os.path.exists(path):
            s=os.stat(path)
            date=s[9]
            return date

        raise DAV_NotFound

    def _get_dav_getcontenttype(self, uri):
        """ find out yourself! """

        path=self.uri2local(uri)
        if os.path.exists(path):
            if os.path.isfile(path):
                if MAGIC_AVAILABLE is False \
                        or self.mimecheck is False:
                    return 'application/octet-stream'
                else:
                    ret, encoding = mimetypes.guess_type(path)

                    # for non mimetype related result we
                    # simply return an appropriate type
                    if ret.find('/')==-1:
                        if ret.find('text')>=0:
                            return 'text/plain'
                        else:
                            return 'application/octet-stream'
                    else:
                        return ret

            elif os.path.isdir(path):
                return "httpd/unix-directory"

        raise DAV_NotFound('Could not find %s' % path)

    def put(self, uri, data, content_type=None):
        """ put the object into the filesystem """
        path=self.uri2local(uri)
        try:
            fp=open(path, "w+")
            if isinstance(data, types.GeneratorType):
                for d in data:
                    fp.write(d)
            else:
                if data:
                    fp.write(data.decode())
            fp.close()
            log.info('put: Created %s' % uri)
        except:
            log.info('put: Could not create %s' % uri)
            raise DAV_Error(424)

        return None

    def mkcol(self,uri):
        """ create a new collection """
        path=self.uri2local(uri)

        # remove trailing slash
        if path[-1]=="/": path=path[:-1]

        # test if file already exists
        if os.path.exists(path):
            raise DAV_Error(405)

        # test if parent exists
        h,t=os.path.split(path)
        if not os.path.exists(h):
            raise DAV_Error(409)

        # test, if we are allowed to create it
        try:
            os.mkdir(path)
            log.info('mkcol: Created new collection %s' % path)
            return 201
        except:
            log.info('mkcol: Creation of %s denied' % path)
            raise DAV_Forbidden

    ### ?? should we do the handler stuff for DELETE, too ?
    ### (see below)

    def rmcol(self,uri):
        """ delete a collection """
        path=self.uri2local(uri)
        if not os.path.exists(path):
            raise DAV_NotFound

        try:
            shutil.rmtree(path)
        except OSError:
            raise DAV_Forbidden # forbidden
        
        return 204

    def rm(self,uri):
        """ delete a normal resource """
        path=self.uri2local(uri)
        if not os.path.exists(path):
            raise DAV_NotFound

        try:
            os.unlink(path)
        except OSError as ex:
            log.info('rm: Forbidden (%s)' % ex)
            raise DAV_Forbidden # forbidden

        return 204

    ###
    ### DELETE handlers (examples)
    ### (we use the predefined methods in davcmd instead of doing
    ### a rm directly
    ###

    def delone(self,uri):
        """ delete a single resource

        You have to return a result dict of the form
        uri:error_code
        or None if everything's ok

        """
        return delone(self,uri)

    def deltree(self,uri):
        """ delete a collection 

        You have to return a result dict of the form
        uri:error_code
        or None if everything's ok
        """

        return deltree(self,uri)


    ###
    ### MOVE handlers (examples)
    ###

    def moveone(self,src,dst,overwrite):
        """ move one resource with Depth=0
        """

        return moveone(self,src,dst,overwrite)

    def movetree(self,src,dst,overwrite):
        """ move a collection with Depth=infinity
        """

        return movetree(self,src,dst,overwrite)

    ###
    ### COPY handlers
    ###

    def copyone(self,src,dst,overwrite):
        """ copy one resource with Depth=0
        """

        return copyone(self,src,dst,overwrite)

    def copytree(self,src,dst,overwrite):
        """ copy a collection with Depth=infinity
        """

        return copytree(self,src,dst,overwrite)

    ###
    ### copy methods.
    ### This methods actually copy something. low-level
    ### They are called by the davcmd utility functions
    ### copytree and copyone (not the above!)
    ### Look in davcmd.py for further details.
    ###

    def copy(self,src,dst):
        """ copy a resource from src to dst """

        srcfile=self.uri2local(src)
        dstfile=self.uri2local(dst)
        try:
            shutil.copy(srcfile, dstfile)
        except (OSError, IOError):
            log.info('copy: forbidden')
            raise DAV_Error(409)

    def copycol(self, src, dst):
        """ copy a collection.

        As this is not recursive (the davserver recurses itself)
        we will only create a new directory here. For some more
        advanced systems we might also have to copy properties from
        the source to the destination.
        """

        return self.mkcol(dst)

    def exists(self,uri):
        """ test if a resource exists """
        path=self.uri2local(uri)
        if os.path.exists(path):
            return 1
        return None

    def is_collection(self,uri):
        """ test if the given uri is a collection """
        path=self.uri2local(uri)
        if os.path.isdir(path):
            return 1
        else:
            return 0



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnu.org>
#    Copyright (C) 2013  Sebastián Marro <smarro@thymbra.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from datetime import datetime
from trytond.model import ModelView, fields
from trytond.wizard import Wizard, StateAction, StateTransition, StateView, \
    Button
from trytond.transaction import Transaction
from trytond.pyson import PYSONEncoder
from trytond.pool import Pool

__all__ = ['WizardGenerateResult', 'RequestImagingTest',
    'RequestPatientImagingTestStart', 'RequestPatientImagingTest']


class WizardGenerateResult(Wizard):
    'Generate Results'
    __name__ = 'wizard.generate.result'
    start_state = 'open_'
    open_ = StateAction('health_imaging.act_imaging_test_result_view')

    def do_open_(self, action):
        pool = Pool()
        Request = pool.get('gnuhealth.imaging.test.request')
        Result = pool.get('gnuhealth.imaging.test.result')

        request_data = []
        requests = Request.browse(Transaction().context.get('active_ids'))
        for request in requests:
            request_data.append({
                'patient': request.patient.id,
                'date': datetime.now(),
                'request_date': request.date,
                'requested_test': request.requested_test,
                'request': request.id,
                'doctor': request.doctor})
        results = Result.create(request_data)

        action['pyson_domain'] = PYSONEncoder().encode(
            [('id', 'in', [r.id for r in results])])

        Request.requested(requests)
        Request.done(requests)
        return action, {}


class RequestImagingTest(ModelView):
    'Request - Test'
    __name__ = 'gnuhealth.request-imaging-test'
    _table = 'gnuhealth_request_imaging_test'

    request = fields.Many2One('gnuhealth.patient.imaging.test.request.start',
        'Request', required=True)
    test = fields.Many2One('gnuhealth.imaging.test', 'Test', required=True)


class RequestPatientImagingTestStart(ModelView):
    'Request Patient Imaging Test Start'
    __name__ = 'gnuhealth.patient.imaging.test.request.start'

    date = fields.DateTime('Date')
    patient = fields.Many2One('gnuhealth.patient', 'Patient', required=True)
    doctor = fields.Many2One('gnuhealth.healthprofessional', 'Doctor',
        required=True, help="Doctor who Request the lab tests.")
    tests = fields.Many2Many('gnuhealth.request-imaging-test', 'request',
        'test', 'Tests', required=True)
    urgent = fields.Boolean('Urgent')

    @staticmethod
    def default_date():
        return datetime.now()

    @staticmethod
    def default_patient():
        if Transaction().context.get('active_model') == 'gnuhealth.patient':
            return Transaction().context.get('active_id')

    @staticmethod
    def default_doctor():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        hp = HealthProf.get_health_professional()
        if not hp:
            RequestPatientImagingTestStart.raise_user_error(
                "No health professional associated to this user !")
        return hp


class RequestPatientImagingTest(Wizard):
    'Request Patient Imaging Test'
    __name__ = 'gnuhealth.patient.imaging.test.request'

    start = StateView('gnuhealth.patient.imaging.test.request.start',
        'health_imaging.patient_imaging_test_request_start_view_form', [
            Button('Cancel', 'end', 'tryton-cancel'),
            Button('Request', 'request', 'tryton-ok', default=True),
            ])
    request = StateTransition()

    def transition_request(self):
        ImagingTestRequest = Pool().get('gnuhealth.imaging.test.request')
        Sequence = Pool().get('ir.sequence')
        Config = Pool().get('gnuhealth.sequences')

        config = Config(1)
        request_number = Sequence.get_id(config.imaging_request_sequence.id)
        imaging_tests = []
        for test in self.start.tests:
            imaging_test = {}
            imaging_test['request'] = request_number
            imaging_test['requested_test'] = test.id
            imaging_test['patient'] = self.start.patient.id
            if self.start.doctor:
                imaging_test['doctor'] = self.start.doctor.id
            imaging_test['date'] = self.start.date
            imaging_test['urgent'] = self.start.urgent
            imaging_tests.append(imaging_test)
        ImagingTestRequest.create(imaging_tests)

        return 'end'



import time
import urllib.request, urllib.parse, urllib.error
import urllib.parse
import random

import logging

log = logging.getLogger(__name__)

import xml.dom
from xml.dom import minidom

from .utils import rfc1123_date, IfParser, tokenFinder
from .errors import *

tokens_to_lock = {}
uris_to_token = {}

class LockManager:
    """ Implements the locking backend and serves as MixIn for DAVRequestHandler """

    def _init_locks(self):
        return tokens_to_lock, uris_to_token

    def _l_isLocked(self, uri):
        tokens, uris = self._init_locks()
        return uri in uris

    def _l_hasLock(self, token):
        tokens, uris = self._init_locks()
        return token in tokens

    def _l_getLockForUri(self, uri):
        tokens, uris = self._init_locks()
        return uris.get(uri, None)

    def _l_getLock(self, token):
        tokens, uris = self._init_locks()
        return tokens.get(token, None)

    def _l_delLock(self, token):
        tokens, uris = self._init_locks()
        if token in tokens:
            del uris[tokens[token].uri]
            del tokens[token]

    def _l_setLock(self, lock):
        tokens, uris = self._init_locks()
        tokens[lock.token] = lock
        uris[lock.uri] = lock

    def _lock_unlock_parse(self, body):
        doc = minidom.parseString(body)

        data = {}
        info = doc.getElementsByTagNameNS('DAV:', 'lockinfo')[0]
        data['lockscope'] = info.getElementsByTagNameNS('DAV:', 'lockscope')[0]\
                                .firstChild.localName
        data['locktype'] = info.getElementsByTagNameNS('DAV:', 'locktype')[0]\
                                .firstChild.localName
        data['lockowner'] = info.getElementsByTagNameNS('DAV:', 'owner')
        return data

    def _lock_unlock_create(self, uri, creator, depth, data):
        lock = LockItem(uri, creator, **data)
        iscollection = uri[-1] == '/' # very dumb collection check

        result = ''
        if depth == 'infinity' and iscollection:
            # locking of children/collections not yet supported
            pass

        if not self._l_isLocked(uri):
            self._l_setLock(lock)

        # because we do not handle children we leave result empty
        return lock.token, result

    def do_UNLOCK(self):
        """ Unlocks given resource """

        dc = self.IFACE_CLASS

        if self._config.DAV.getboolean('verbose') is True:
            log.info('UNLOCKing resource %s' % self.headers)

        uri = urllib.parse.urljoin(self.get_baseuri(dc), self.path)
        uri = urllib.parse.unquote(uri)

        # check lock token - must contain a dash
        if not self.headers.get('Lock-Token', '').find('-')>0:
            return self.send_status(400)

        token = tokenFinder(self.headers.get('Lock-Token'))
        if self._l_isLocked(uri):
            self._l_delLock(token)

        self.send_body(None, '204', 'Ok', 'Ok')

    def do_LOCK(self):
        """ Locking is implemented via in-memory caches. No data is written to disk.  """

        dc = self.IFACE_CLASS

        log.info('LOCKing resource %s' % self.headers)

        body = None
        if 'Content-Length' in self.headers:
            l = self.headers['Content-Length']
            body = self.rfile.read(int(l))

        depth = self.headers.get('Depth', 'infinity')

        uri = urllib.parse.urljoin(self.get_baseuri(dc), self.path)
        uri = urllib.parse.unquote(uri)
        log.info('do_LOCK: uri = %s' % uri)

        ifheader = self.headers.get('If')
        alreadylocked = self._l_isLocked(uri)
        log.info('do_LOCK: alreadylocked = %s' % alreadylocked)

        if body and alreadylocked:
            # Full LOCK request but resource already locked
            self.responses[423] = ('Locked', 'Already locked')
            return self.send_status(423)

        elif body and not ifheader:
            # LOCK with XML information
            data = self._lock_unlock_parse(body)
            token, result = self._lock_unlock_create(uri, 'unknown', depth, data)

            if result:
                self.send_body(result, '207', 'Error', 'Error',
                                'text/xml; charset="utf-8"')

            else:
                lock = self._l_getLock(token)
                self.send_body(lock.asXML(), '200', 'OK', 'OK',
                                'text/xml; charset="utf-8"',
                                {'Lock-Token' : '<opaquelocktoken:%s>' % token})


        else:
            # refresh request - refresh lock timeout
            taglist = IfParser(ifheader)
            found = 0
            for tag in taglist:
                for listitem in tag.list:
                    token = tokenFinder(listitem)
                    if token and self._l_hasLock(token):
                        lock = self._l_getLock(token)
                        timeout = self.headers.get('Timeout', 'Infinite')
                        lock.setTimeout(timeout) # automatically refreshes
                        found = 1

                        self.send_body(lock.asXML(), 
                                        '200', 'OK', 'OK', 'text/xml; encoding="utf-8"')
                        break
                if found: 
                    break

            # we didn't find any of the tokens mentioned - means
            # that table was cleared or another error
            if not found:
                self.send_status(412) # precondition failed

class LockItem:
    """ Lock with support for exclusive write locks. Some code taken from
    webdav.LockItem from the Zope project. """

    def __init__(self, uri, creator, lockowner, depth=0, timeout='Infinite',
                    locktype='write', lockscope='exclusive', token=None, **kw):

        self.uri = uri
        self.creator = creator
        self.owner = lockowner
        self.depth = depth
        self.timeout = timeout
        self.locktype = locktype
        self.lockscope = lockscope
        self.token = token and token or self.generateToken()
        self.modified = time.time()

    def getModifiedTime(self):
        return self.modified

    def refresh(self):
        self.modified = time.time()

    def isValid(self):
        now = time.time()
        modified = self.modified
        timeout = self.timeout
        return (modified + timeout) > now

    def generateToken(self):
        _randGen = random.Random(time.time())
        return '%s-%s-00105A989226:%.03f' %  \
             (_randGen.random(),_randGen.random(),time.time())

    def getTimeoutString(self):
        t = str(self.timeout)
        if t[-1] == 'L': t = t[:-1]
        return 'Second-%s' % t

    def setTimeout(self, timeout):
        self.timeout = timeout
        self.modified = time.time()

    def asXML(self, namespace='d', discover=False):
        owner_str = ''
        if isinstance(self.owner, str):
            owner_str = self.owner
        elif isinstance(self.owner, xml.dom.minicompat.NodeList):
            owner_str = "".join([node.toxml() for node in self.owner[0].childNodes])

        token = self.token
        base = ('<%(ns)s:activelock>\n'
             '  <%(ns)s:locktype><%(ns)s:%(locktype)s/></%(ns)s:locktype>\n'
             '  <%(ns)s:lockscope><%(ns)s:%(lockscope)s/></%(ns)s:lockscope>\n'
             '  <%(ns)s:depth>%(depth)s</%(ns)s:depth>\n'
             '  <%(ns)s:owner>%(owner)s</%(ns)s:owner>\n'
             '  <%(ns)s:timeout>%(timeout)s</%(ns)s:timeout>\n'
             '  <%(ns)s:locktoken>\n'
             '   <%(ns)s:href>opaquelocktoken:%(locktoken)s</%(ns)s:href>\n'
             '  </%(ns)s:locktoken>\n'
             ' </%(ns)s:activelock>\n'
             ) % {
               'ns': namespace,
               'locktype': self.locktype,
               'lockscope': self.lockscope,
               'depth': self.depth,
               'owner': owner_str,
               'timeout': self.getTimeoutString(),
               'locktoken': token,
               }

        if discover is True:
            return base

        s = """<?xml version="1.0" encoding="utf-8" ?>
<d:prop xmlns:d="DAV:">
 <d:lockdiscovery>
  %s
 </d:lockdiscovery>
</d:prop>""" % base

        return s



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from datetime import datetime
from trytond.model import ModelView, ModelSQL, fields
from trytond.transaction import Transaction
from trytond.rpc import RPC
from trytond.pool import Pool
from trytond.wizard import Wizard, StateAction, StateView, Button
from trytond.pyson import Eval, Not, Bool, PYSONEncoder, Equal, And, Or
import hashlib
import json

__all__ = ['LabTest']


class LabTest(ModelSQL, ModelView):
    'Lab Test'
    __name__ = 'gnuhealth.lab'

    STATES = {'readonly': Eval('state') == 'validated'}

    serializer = fields.Text('Doc String', readonly=True)

    document_digest = fields.Char('Digest', readonly=True,
        help="Original Document Digest")

    state = fields.Selection([
        ('draft', 'Draft'),
        ('done', 'Done'),        
        ('validated', 'Validated'),        
        ], 'State', readonly=True, sort=False)


    digest_status = fields.Function(fields.Boolean('Altered',
        help="This field will be set whenever parts of" \
        " the main original document has been changed." \
        " Please note that the verification is done only on selected" \
        " fields." ),
        'check_digest')

    serializer_current = fields.Function(fields.Text('Current Doc',
            states={
            'invisible': Not(Bool(Eval('digest_status'))),
            }),
        'check_digest')

        
    digest_current = fields.Function(fields.Char('Current Hash',
            states={
            'invisible': Not(Bool(Eval('digest_status'))),
            }),
        'check_digest')

    digital_signature = fields.Text('Digital Signature', readonly=True)

    done_by = fields.Many2One(
        'gnuhealth.healthprofessional', 
        'Done by', readonly=True, help='Professional who processes this'
        ' lab test',
        states = STATES )

    done_date = fields.DateTime('Finished on', readonly=True,
        states = STATES )

    validated_by = fields.Many2One(
        'gnuhealth.healthprofessional', 
        'Validated by', readonly=True, help='Professional who validates this'
        ' lab test',
        states = STATES )

    validation_date = fields.DateTime('Validated on', readonly=True,
        states = STATES )
        
    @staticmethod
    def default_state():
        return 'draft'

    @classmethod
    def __setup__(cls):
        super(LabTest, cls).__setup__()
        cls._buttons.update({
            'generate_document': {
                'invisible': Not(Equal(Eval('state'), 'draft')),
                },
            'set_to_draft': {
                'invisible': Not(Equal(Eval('state'), 'done')),
                },
            'sign_document': {
                'invisible': Not(Equal(Eval('state'), 'done')),
                },
            })
        ''' Allow calling the set_signature method via RPC '''
        cls.__rpc__.update({
                'set_signature': RPC(readonly=False),
                })

    @classmethod
    @ModelView.button
    def generate_document(cls, documents):
        pool = Pool()
        HealthProfessional = pool.get('gnuhealth.healthprofessional')

        document = documents[0]


        # Set the document to "Done"
        # and write the name of the signing health professional

        hp = HealthProfessional.get_health_professional()
        if not hp:
            cls.raise_user_error(
                "No health professional associated to this user !")

        serial_doc=cls.get_serial(document)

        cls.write(documents, {
            'done_by': hp,
            'done_date': datetime.now(),
            'state': 'done',})

    @classmethod
    @ModelView.button
    def set_to_draft(cls, documents):
        document = documents[0]

        cls.write(documents, {
            'state': 'draft',})

    @classmethod
    @ModelView.button
    def sign_document(cls, documents):
        pool = Pool()
        HealthProfessional = pool.get('gnuhealth.healthprofessional')

        document = documents[0]

        # Validate / generate digest for the document
        # and write the name of the signing health professional

        hp = HealthProfessional.get_health_professional()
        if not hp:
            cls.raise_user_error(
                "No health professional associated to this user !")

        serial_doc=cls.get_serial(document)
        
        cls.write(documents, {
            'serializer': serial_doc,
            'document_digest': HealthCrypto().gen_hash(serial_doc),
            'validated_by': hp,
            'validation_date': datetime.now(),
            'state': 'validated',})


    @classmethod
    def get_serial(cls,document):

        analyte_line=[]
        
        for line in document.critearea:
            line_elements=[line.name or '',
                line.result or '', 
                line.result_text or '',
                line.remarks or '']
                
            analyte_line.append(line_elements)

        data_to_serialize = { 
            'Lab_test': str(document.name) or '',
            'Test': str(document.test.rec_name) or '',
            'HP': str(document.requestor.rec_name),
            'Patient': str(document.patient.rec_name),
            'Patient_ID': str(document.patient.name.ref) or '',
            'Analyte_line': str(analyte_line),
             }

        serialized_doc = str(HealthCrypto().serialize(data_to_serialize))
        
        return serialized_doc
    
    @classmethod
    def set_signature(cls, data, signature):
        """
        Set the clearsigned signature
        """

        doc_id = data['id']
        
        cls.write([cls(doc_id)], {
            'digital_signature': signature,
            })


    def check_digest (self,name):
        result=''
        serial_doc=self.get_serial(self)
        if (name == 'digest_status' and self.document_digest):
            if (HealthCrypto().gen_hash(serial_doc) == self.document_digest):
                result = False
            else:
                ''' Return true if the document has been altered'''
                result = True
        if (name=='digest_current'):
            result = HealthCrypto().gen_hash(serial_doc)

        if (name=='serializer_current'):
            result = serial_doc
            
        return result
 
    # Hide the group holding validation information when state is 
    # not validated
    
    @classmethod
    def view_attributes(cls):
        return [('//group[@id="document_digest"]', 'states', {
                'invisible': Not(Eval('state') == 'validated'),
                })]
       



class HealthCrypto:
    """ GNU Health Cryptographic functions
    """

    def serialize(self,data_to_serialize):
        """ Format to JSON """

        json_output = \
            json.dumps(data_to_serialize,ensure_ascii=False)
        return json_output

    def gen_hash(self, serialized_doc):
        return hashlib.sha512(serialized_doc.encode('utf-8')).hexdigest()





class familyMember:
    contents=[{'code': 'FAMMEMB',
          'display': 'member',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'CHILD',
          'display': 'child',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'CHLDADOPT',
          'display': 'adopted child',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'DAUADOPT',
          'display': 'adopted daughter',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'SONADOPT',
          'display': 'adopted son',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'CHLDFOST',
          'display': 'foster child',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'DAUFOST',
          'display': 'foster daughter',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'SONFOST',
          'display': 'foster son',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'DAUC',
          'display': 'daughter',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'DAU',
          'display': 'natural daughter',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'STPDAU',
          'display': 'stepdaughter',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'NCHILD',
          'display': 'natural child',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'SON',
          'display': 'natural son',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'SONC',
          'display': 'son',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'STPSON',
          'display': 'stepson',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'STPCHLD',
          'display': 'step child',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'EXT',
          'display': 'extended family member',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'AUNT',
          'display': 'aunt',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'MAUNT',
          'display': 'maternal aunt',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'PAUNT',
          'display': 'paternal aunt',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'COUSN',
          'display': 'cousin',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'MCOUSN',
          'display': 'maternal cousin',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'PCOUSN',
          'display': 'paternal cousin',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'GGRPRN',
          'display': 'great grandparent',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'GGRFTH',
          'display': 'great grandfather',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'MGGRFTH',
          'display': 'maternal great-grandfather',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'PGGRFTH',
          'display': 'paternal great-grandfather',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'GGRMTH',
          'display': 'great grandmother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'MGGRMTH',
          'display': 'maternal great-grandmother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'PGGRMTH',
          'display': 'paternal great-grandmother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'MGGRPRN',
          'display': 'maternal great-grandparent',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'PGGRPRN',
          'display': 'paternal great-grandparent',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'GRNDCHILD',
          'display': 'grandchild',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'GRNDDAU',
          'display': 'granddaughter',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'GRNDSON',
          'display': 'grandson',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'GRPRN',
          'display': 'grandparent',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'GRFTH',
          'display': 'grandfather',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'MGRFTH',
          'display': 'maternal grandfather',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'PGRFTH',
          'display': 'paternal grandfather',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'GRMTH',
          'display': 'grandmother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'MGRMTH',
          'display': 'maternal grandmother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'PGRMTH',
          'display': 'paternal grandmother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'MGRPRN',
          'display': 'maternal grandparent',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'PGRPRN',
          'display': 'paternal grandparent',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'INLAW',
          'display': 'inlaw',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'CHLDINLAW',
          'display': 'child-in-law',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'DAUINLAW',
          'display': 'daughter in-law',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'SONINLAW',
          'display': 'son in-law',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'PRNINLAW',
          'display': 'parent in-law',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'FTHINLAW',
          'display': 'father-in-law',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'MTHINLAW',
          'display': 'mother-in-law',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'SIBINLAW',
          'display': 'sibling in-law',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'BROINLAW',
          'display': 'brother-in-law',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'SISINLAW',
          'display': 'sister-in-law',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'NIENEPH',
          'display': 'niece/nephew',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'NEPHEW',
          'display': 'nephew',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'NIECE',
          'display': 'niece',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'UNCLE',
          'display': 'uncle',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'MUNCLE',
          'display': 'maternal uncle',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'PUNCLE',
          'display': 'paternal uncle',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'PRN',
          'display': 'parent',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'ADOPTP',
          'display': 'adoptive parent',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'ADOPTF',
          'display': 'adoptive father',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'ADOPTM',
          'display': 'adoptive mother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'FTH',
          'display': 'father',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'FTHFOST',
          'display': 'foster father',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'NFTH',
          'display': 'natural father',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'NFTHF',
          'display': 'natural father of fetus',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'STPFTH',
          'display': 'stepfather',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'MTH',
          'display': 'mother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'GESTM',
          'display': 'gestational mother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'MTHFOST',
          'display': 'foster mother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'NMTH',
          'display': 'natural mother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'NMTHF',
          'display': 'natural mother of fetus',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'STPMTH',
          'display': 'stepmother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'NPRN',
          'display': 'natural parent',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'PRNFOST',
          'display': 'foster parent',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'STPPRN',
          'display': 'step parent',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'SIB',
          'display': 'sibling',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'BRO',
          'display': 'brother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'HBRO',
          'display': 'half-brother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'NBRO',
          'display': 'natural brother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'TWINBRO',
          'display': 'twin brother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'FTWINBRO',
          'display': 'fraternal twin brother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'ITWINBRO',
          'display': 'identical twin brother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'STPBRO',
          'display': 'stepbrother',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'HSIB',
          'display': 'half-sibling',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'HSIS',
          'display': 'half-sister',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'NSIB',
          'display': 'natural sibling',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'NSIS',
          'display': 'natural sister',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'TWINSIS',
          'display': 'twin sister',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'FTWINSIS',
          'display': 'fraternal twin sister',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'ITWINSIS',
          'display': 'identical twin sister',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'TWIN',
          'display': 'twin',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'FTWIN',
          'display': 'fraternal twin',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'ITWIN',
          'display': 'identical twin',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'SIS',
          'display': 'sister',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'STPSIS',
          'display': 'stepsister',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'STPSIB',
          'display': 'step sibling',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'SIGOTHR',
          'display': 'significant other',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'DOMPART',
          'display': 'domestic partner',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'SPS',
          'display': 'spouse',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'HUSB',
          'display': 'husband',
          'system': 'http://hl7.org/fhir/v3/RoleCode'},
         {'code': 'WIFE',
          'display': 'wife',
          'system': 'http://hl7.org/fhir/v3/RoleCode'}]
__all__=['familyMember']



from .base_classes import *
from .support_functions import *


class MessageHeader(Resource):
    """The header for a message exchange that is either requesting or
    responding to an action. The resource(s) that are the subject of
    the action as well as other Information related to the action
    are typically transmitted in a bundle in which the MessageHeader
    resource instance is the first resource in the bundle.If the
    element is present, it must have either a @value, an @id, or
    extensions"""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, timestamp=None, event=None, response=None, source=None, destination=None, enterer=None, author=None, receiver=None, responsible=None, reason=None, data=None):
        self.original_tagname_ = None
        super(MessageHeader, self).__init__(id, extension, modifierExtension, language, text, contained, )
        self.identifier = identifier
        self.timestamp = timestamp
        self.event = event
        self.response = response
        self.source = source
        if destination is None:
            self.destination = []
        else:
            self.destination = destination
        self.enterer = enterer
        self.author = author
        self.receiver = receiver
        self.responsible = responsible
        self.reason = reason
        if data is None:
            self.data = []
        else:
            self.data = data
    def factory(*args_, **kwargs_):
        if MessageHeader.subclass:
            return MessageHeader.subclass(*args_, **kwargs_)
        else:
            return MessageHeader(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def get_timestamp(self): return self.timestamp
    def set_timestamp(self, timestamp): self.timestamp = timestamp
    def get_event(self): return self.event
    def set_event(self, event): self.event = event
    def get_response(self): return self.response
    def set_response(self, response): self.response = response
    def get_source(self): return self.source
    def set_source(self, source): self.source = source
    def get_destination(self): return self.destination
    def set_destination(self, destination): self.destination = destination
    def add_destination(self, value): self.destination.append(value)
    def insert_destination(self, index, value): self.destination[index] = value
    def get_enterer(self): return self.enterer
    def set_enterer(self, enterer): self.enterer = enterer
    def get_author(self): return self.author
    def set_author(self, author): self.author = author
    def get_receiver(self): return self.receiver
    def set_receiver(self, receiver): self.receiver = receiver
    def get_responsible(self): return self.responsible
    def set_responsible(self, responsible): self.responsible = responsible
    def get_reason(self): return self.reason
    def set_reason(self, reason): self.reason = reason
    def get_data(self): return self.data
    def set_data(self, data): self.data = data
    def add_data(self, value): self.data.append(value)
    def insert_data(self, index, value): self.data[index] = value
    def hasContent_(self):
        if (
            self.identifier is not None or
            self.timestamp is not None or
            self.event is not None or
            self.response is not None or
            self.source is not None or
            self.destination or
            self.enterer is not None or
            self.author is not None or
            self.receiver is not None or
            self.responsible is not None or
            self.reason is not None or
            self.data or
            super(MessageHeader, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MessageHeader', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MessageHeader')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MessageHeader', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MessageHeader'):
        super(MessageHeader, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MessageHeader')
    def exportChildren(self, outfile, level, namespace_='', name_='MessageHeader', fromsubclass_=False, pretty_print=True):
        super(MessageHeader, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.identifier is not None:
            self.identifier.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.timestamp is not None:
            self.timestamp.export(outfile, level, namespace_, name_='timestamp', pretty_print=pretty_print)
        if self.event is not None:
            self.event.export(outfile, level, namespace_, name_='event', pretty_print=pretty_print)
        if self.response is not None:
            self.response.export(outfile, level, namespace_, name_='response', pretty_print=pretty_print)
        if self.source is not None:
            self.source.export(outfile, level, namespace_, name_='source', pretty_print=pretty_print)
        for destination_ in self.destination:
            destination_.export(outfile, level, namespace_, name_='destination', pretty_print=pretty_print)
        if self.enterer is not None:
            self.enterer.export(outfile, level, namespace_, name_='enterer', pretty_print=pretty_print)
        if self.author is not None:
            self.author.export(outfile, level, namespace_, name_='author', pretty_print=pretty_print)
        if self.receiver is not None:
            self.receiver.export(outfile, level, namespace_, name_='receiver', pretty_print=pretty_print)
        if self.responsible is not None:
            self.responsible.export(outfile, level, namespace_, name_='responsible', pretty_print=pretty_print)
        if self.reason is not None:
            self.reason.export(outfile, level, namespace_, name_='reason', pretty_print=pretty_print)
        for data_ in self.data:
            data_.export(outfile, level, namespace_, name_='data', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MessageHeader', mapping_=None):
        element = super(MessageHeader, self).to_etree(parent_element, name_, mapping_)
        if self.identifier is not None:
            identifier_ = self.identifier
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.timestamp is not None:
            timestamp_ = self.timestamp
            timestamp_.to_etree(element, name_='timestamp', mapping_=mapping_)
        if self.event is not None:
            event_ = self.event
            event_.to_etree(element, name_='event', mapping_=mapping_)
        if self.response is not None:
            response_ = self.response
            response_.to_etree(element, name_='response', mapping_=mapping_)
        if self.source is not None:
            source_ = self.source
            source_.to_etree(element, name_='source', mapping_=mapping_)
        for destination_ in self.destination:
            destination_.to_etree(element, name_='destination', mapping_=mapping_)
        if self.enterer is not None:
            enterer_ = self.enterer
            enterer_.to_etree(element, name_='enterer', mapping_=mapping_)
        if self.author is not None:
            author_ = self.author
            author_.to_etree(element, name_='author', mapping_=mapping_)
        if self.receiver is not None:
            receiver_ = self.receiver
            receiver_.to_etree(element, name_='receiver', mapping_=mapping_)
        if self.responsible is not None:
            responsible_ = self.responsible
            responsible_.to_etree(element, name_='responsible', mapping_=mapping_)
        if self.reason is not None:
            reason_ = self.reason
            reason_.to_etree(element, name_='reason', mapping_=mapping_)
        for data_ in self.data:
            data_.to_etree(element, name_='data', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MessageHeader, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = id.factory()
            obj_.build(child_)
            self.identifier = obj_
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'timestamp':
            obj_ = instant.factory()
            obj_.build(child_)
            self.timestamp = obj_
            obj_.original_tagname_ = 'timestamp'
        elif nodeName_ == 'event':
            obj_ = Coding.factory()
            obj_.build(child_)
            self.event = obj_
            obj_.original_tagname_ = 'event'
        elif nodeName_ == 'response':
            obj_ = MessageHeader_Response.factory()
            obj_.build(child_)
            self.response = obj_
            obj_.original_tagname_ = 'response'
        elif nodeName_ == 'source':
            obj_ = MessageHeader_Source.factory()
            obj_.build(child_)
            self.source = obj_
            obj_.original_tagname_ = 'source'
        elif nodeName_ == 'destination':
            obj_ = MessageHeader_Destination.factory()
            obj_.build(child_)
            self.destination.append(obj_)
            obj_.original_tagname_ = 'destination'
        elif nodeName_ == 'enterer':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.enterer = obj_
            obj_.original_tagname_ = 'enterer'
        elif nodeName_ == 'author':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.author = obj_
            obj_.original_tagname_ = 'author'
        elif nodeName_ == 'receiver':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.receiver = obj_
            obj_.original_tagname_ = 'receiver'
        elif nodeName_ == 'responsible':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.responsible = obj_
            obj_.original_tagname_ = 'responsible'
        elif nodeName_ == 'reason':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.reason = obj_
            obj_.original_tagname_ = 'reason'
        elif nodeName_ == 'data':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.data.append(obj_)
            obj_.original_tagname_ = 'data'
        super(MessageHeader, self).buildChildren(child_, node, nodeName_, True)
# end class MessageHeader


class MessageHeader_Response(BackboneElement):
    """The header for a message exchange that is either requesting or
    responding to an action. The resource(s) that are the subject of
    the action as well as other Information related to the action
    are typically transmitted in a bundle in which the MessageHeader
    resource instance is the first resource in the bundle."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, identifier=None, code=None, details=None):
        self.original_tagname_ = None
        super(MessageHeader_Response, self).__init__(id, extension, modifierExtension, )
        self.identifier = identifier
        self.code = code
        self.details = details
    def factory(*args_, **kwargs_):
        if MessageHeader_Response.subclass:
            return MessageHeader_Response.subclass(*args_, **kwargs_)
        else:
            return MessageHeader_Response(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_details(self): return self.details
    def set_details(self, details): self.details = details
    def hasContent_(self):
        if (
            self.identifier is not None or
            self.code is not None or
            self.details is not None or
            super(MessageHeader_Response, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MessageHeader.Response', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MessageHeader.Response')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MessageHeader.Response', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MessageHeader.Response'):
        super(MessageHeader_Response, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MessageHeader.Response')
    def exportChildren(self, outfile, level, namespace_='', name_='MessageHeader.Response', fromsubclass_=False, pretty_print=True):
        super(MessageHeader_Response, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.identifier is not None:
            self.identifier.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.details is not None:
            self.details.export(outfile, level, namespace_, name_='details', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MessageHeader.Response', mapping_=None):
        element = super(MessageHeader_Response, self).to_etree(parent_element, name_, mapping_)
        if self.identifier is not None:
            identifier_ = self.identifier
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.details is not None:
            details_ = self.details
            details_.to_etree(element, name_='details', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MessageHeader_Response, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = id.factory()
            obj_.build(child_)
            self.identifier = obj_
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'code':
            obj_ = ResponseType.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'details':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.details = obj_
            obj_.original_tagname_ = 'details'
        super(MessageHeader_Response, self).buildChildren(child_, node, nodeName_, True)
# end class MessageHeader_Response


class MessageHeader_Source(BackboneElement):
    """The header for a message exchange that is either requesting or
    responding to an action. The resource(s) that are the subject of
    the action as well as other Information related to the action
    are typically transmitted in a bundle in which the MessageHeader
    resource instance is the first resource in the bundle."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, name=None, software=None, version=None, contact=None, endpoint=None):
        self.original_tagname_ = None
        super(MessageHeader_Source, self).__init__(id, extension, modifierExtension, )
        self.name = name
        self.software = software
        self.version = version
        self.contact = contact
        self.endpoint = endpoint
    def factory(*args_, **kwargs_):
        if MessageHeader_Source.subclass:
            return MessageHeader_Source.subclass(*args_, **kwargs_)
        else:
            return MessageHeader_Source(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_software(self): return self.software
    def set_software(self, software): self.software = software
    def get_version(self): return self.version
    def set_version(self, version): self.version = version
    def get_contact(self): return self.contact
    def set_contact(self, contact): self.contact = contact
    def get_endpoint(self): return self.endpoint
    def set_endpoint(self, endpoint): self.endpoint = endpoint
    def hasContent_(self):
        if (
            self.name is not None or
            self.software is not None or
            self.version is not None or
            self.contact is not None or
            self.endpoint is not None or
            super(MessageHeader_Source, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MessageHeader.Source', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MessageHeader.Source')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MessageHeader.Source', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MessageHeader.Source'):
        super(MessageHeader_Source, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MessageHeader.Source')
    def exportChildren(self, outfile, level, namespace_='', name_='MessageHeader.Source', fromsubclass_=False, pretty_print=True):
        super(MessageHeader_Source, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        if self.software is not None:
            self.software.export(outfile, level, namespace_, name_='software', pretty_print=pretty_print)
        if self.version is not None:
            self.version.export(outfile, level, namespace_, name_='version', pretty_print=pretty_print)
        if self.contact is not None:
            self.contact.export(outfile, level, namespace_, name_='contact', pretty_print=pretty_print)
        if self.endpoint is not None:
            self.endpoint.export(outfile, level, namespace_, name_='endpoint', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MessageHeader.Source', mapping_=None):
        element = super(MessageHeader_Source, self).to_etree(parent_element, name_, mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        if self.software is not None:
            software_ = self.software
            software_.to_etree(element, name_='software', mapping_=mapping_)
        if self.version is not None:
            version_ = self.version
            version_.to_etree(element, name_='version', mapping_=mapping_)
        if self.contact is not None:
            contact_ = self.contact
            contact_.to_etree(element, name_='contact', mapping_=mapping_)
        if self.endpoint is not None:
            endpoint_ = self.endpoint
            endpoint_.to_etree(element, name_='endpoint', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MessageHeader_Source, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'name':
            obj_ = string.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'software':
            obj_ = string.factory()
            obj_.build(child_)
            self.software = obj_
            obj_.original_tagname_ = 'software'
        elif nodeName_ == 'version':
            obj_ = string.factory()
            obj_.build(child_)
            self.version = obj_
            obj_.original_tagname_ = 'version'
        elif nodeName_ == 'contact':
            obj_ = Contact.factory()
            obj_.build(child_)
            self.contact = obj_
            obj_.original_tagname_ = 'contact'
        elif nodeName_ == 'endpoint':
            obj_ = uri.factory()
            obj_.build(child_)
            self.endpoint = obj_
            obj_.original_tagname_ = 'endpoint'
        super(MessageHeader_Source, self).buildChildren(child_, node, nodeName_, True)
# end class MessageHeader_Source


class MessageHeader_Destination(BackboneElement):
    """The header for a message exchange that is either requesting or
    responding to an action. The resource(s) that are the subject of
    the action as well as other Information related to the action
    are typically transmitted in a bundle in which the MessageHeader
    resource instance is the first resource in the bundle."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, name=None, target=None, endpoint=None):
        self.original_tagname_ = None
        super(MessageHeader_Destination, self).__init__(id, extension, modifierExtension, )
        self.name = name
        self.target = target
        self.endpoint = endpoint
    def factory(*args_, **kwargs_):
        if MessageHeader_Destination.subclass:
            return MessageHeader_Destination.subclass(*args_, **kwargs_)
        else:
            return MessageHeader_Destination(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_name(self): return self.name
    def set_name(self, name): self.name = name
    def get_target(self): return self.target
    def set_target(self, target): self.target = target
    def get_endpoint(self): return self.endpoint
    def set_endpoint(self, endpoint): self.endpoint = endpoint
    def hasContent_(self):
        if (
            self.name is not None or
            self.target is not None or
            self.endpoint is not None or
            super(MessageHeader_Destination, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='MessageHeader.Destination', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='MessageHeader.Destination')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='MessageHeader.Destination', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='MessageHeader.Destination'):
        super(MessageHeader_Destination, self).exportAttributes(outfile, level, already_processed, namespace_, name_='MessageHeader.Destination')
    def exportChildren(self, outfile, level, namespace_='', name_='MessageHeader.Destination', fromsubclass_=False, pretty_print=True):
        super(MessageHeader_Destination, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.name is not None:
            self.name.export(outfile, level, namespace_, name_='name', pretty_print=pretty_print)
        if self.target is not None:
            self.target.export(outfile, level, namespace_, name_='target', pretty_print=pretty_print)
        if self.endpoint is not None:
            self.endpoint.export(outfile, level, namespace_, name_='endpoint', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='MessageHeader.Destination', mapping_=None):
        element = super(MessageHeader_Destination, self).to_etree(parent_element, name_, mapping_)
        if self.name is not None:
            name_ = self.name
            name_.to_etree(element, name_='name', mapping_=mapping_)
        if self.target is not None:
            target_ = self.target
            target_.to_etree(element, name_='target', mapping_=mapping_)
        if self.endpoint is not None:
            endpoint_ = self.endpoint
            endpoint_.to_etree(element, name_='endpoint', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(MessageHeader_Destination, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'name':
            obj_ = string.factory()
            obj_.build(child_)
            self.name = obj_
            obj_.original_tagname_ = 'name'
        elif nodeName_ == 'target':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.target = obj_
            obj_.original_tagname_ = 'target'
        elif nodeName_ == 'endpoint':
            obj_ = uri.factory()
            obj_.build(child_)
            self.endpoint = obj_
            obj_.original_tagname_ = 'endpoint'
        super(MessageHeader_Destination, self).buildChildren(child_, node, nodeName_, True)
# end class MessageHeader_Destination


class ResponseType(Element):
    """The kind of response to a messageIf the element is present, it must
    have either a @value, an @id, or extensions"""
    subclass = None
    superclass = Element
    def __init__(self, id=None, extension=None, value=None):
        self.original_tagname_ = None
        super(ResponseType, self).__init__(id, extension, )
        self.value = _cast(None, value)
    def factory(*args_, **kwargs_):
        if ResponseType.subclass:
            return ResponseType.subclass(*args_, **kwargs_)
        else:
            return ResponseType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_value(self): return self.value
    def set_value(self, value): self.value = value
    def hasContent_(self):
        if (
            super(ResponseType, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='ResponseType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='ResponseType')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='ResponseType', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='ResponseType'):
        super(ResponseType, self).exportAttributes(outfile, level, already_processed, namespace_, name_='ResponseType')
        if self.value is not None and 'value' not in already_processed:
            already_processed.add('value')
            outfile.write(' value=%s' % (quote_attrib(self.value), ))
    def exportChildren(self, outfile, level, namespace_='', name_='ResponseType', fromsubclass_=False, pretty_print=True):
        super(ResponseType, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='ResponseType', mapping_=None):
        element = super(ResponseType, self).to_etree(parent_element, name_, mapping_)
        if self.value is not None:
            element.set('value', self.value)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('value', node)
        if value is not None and 'value' not in already_processed:
            already_processed.add('value')
            self.value = value
        super(ResponseType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        super(ResponseType, self).buildChildren(child_, node, nodeName_, True)
        pass
# end class ResponseType



import server.fhir as supermod
from .health_fhir_patient import *
from .health_fhir_atom import *
from .health_fhir_search import health_Search
from .health_fhir_practitioner import *
from .health_fhir_operation_outcome import *
from .health_fhir_observation import *
from .health_fhir_procedure import *
from .health_fhir_diagnostic_report import *
from .health_fhir_conformance import *
from .health_fhir_condition import *
from .health_fhir_family_history import *
from .health_fhir_medication import *
from .health_fhir_medication_statement import *
from .health_fhir_immunization import *
from .health_fhir_organization import *
from .datastore import *
from lxml import etree as etree_

def get_root_tag(node):
    tag = supermod.Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = supermod.GDSClassesMapping.get(tag)
    if rootClass is None and hasattr(supermod, tag):
        rootClass = getattr(supermod, tag)
    return tag, rootClass

def parse(inFileName, silence=False):
    doc = supermod.parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Element'
        rootClass = supermod.Element
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    if not silence:
        sys.stdout.write('<?xml version="1.0" ?>\n')
        rootObj.export(
            sys.stdout, 0, name_=rootTag,
            namespacedef_='',
            pretty_print=True)
    return rootObj

def parseEtree(inFilename, silence=False):
    doc = supermod.parsexml_(inFilename)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Element'
        rootClass = supermod.Element
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    mapping = {}
    rootElement = rootObj.to_etree(None, name_=rootTag, mapping_=mapping)
    reverse_mapping = rootObj.gds_reverse_node_mapping(mapping)
    if not silence:
        content = etree_.tostring(
            rootElement, pretty_print=True,
            xml_declaration=True, encoding="utf-8")
        sys.stdout.write(content)
        sys.stdout.write('\n')
    return rootObj, rootElement, mapping, reverse_mapping

def parseString(inString, silence=False):
    from StringIO import StringIO
    doc = supermod.parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Element'
        rootClass = supermod.Element
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    if not silence:
        sys.stdout.write('<?xml version="1.0" ?>\n')
        rootObj.export(
            sys.stdout, 0, name_=rootTag,
            namespacedef_='')
    return rootObj

def parseLiteral(inFilename, silence=False):
    doc = supermod.parsexml_(inFilename)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Element'
        rootClass = supermod.Element
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    if not silence:
        sys.stdout.write('#from ??? import *\n\n')
        sys.stdout.write('import ??? as model_\n\n')
        sys.stdout.write('rootObj = model_.rootClass(\n')
        rootObj.exportLiteral(sys.stdout, 0, name_=rootTag)
        sys.stdout.write(')\n')
    return rootObj
